/****************************************************************************
* Title                 :   STEPPER LIBRARY
* Filename              :   stepper_lib.h
* Author                :   MSV
* Origin Date           :   28/01/2016
* Notes                 :   None
*****************************************************************************/
/**************************CHANGE LIST **************************************
*
*    Date    Software Version    Initials   Description
*  28/01/16    XXXXXXXXXXX         MSV      Interface Created.
*
*****************************************************************************/
/**
 * @file stepper_lib.h
 * @brief <h3> Stepper Library </h3>
 *
 * @par
 * Low level functions for
 * <a href="http://www.mikroe.com">MikroElektronika's</a> CLICKNAME click
 * board.
 */

/**
 * @page LIB_INFO Library Info
 * @date 28 Jan 2016
 * @author Milos Vidojevic
 * @copyright GNU Public License
 * @version 1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :      STM32F107VC
 * -<b> Dev. Board    </b> :      EasyMx Pro v7
 * -<b> Oscillator    </b> :      72 Mhz internal
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for ARM 4.7.1
 *
 * ### Test configuration PIC32 : ###
 * @par
 * -<b> MCU           </b> :      PIC32MX795F512L
 * -<b> Dev. Board    </b> :      EasyPIC Fusion v7
 * -<b> Oscillator    </b> :      80 Mhz internal
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for PIC 3.6.0
 *
 * ### Test configuration FT90x : ###
 * @par
 * -<b> MCU           </b> :      FT900Q
 * -<b> Dev. Board    </b> :      EasyFT90x v7
 * -<b> Oscillator    </b> :      100 Mhz internal
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for FT90x 1.2.1
 *
 * ### Test configuration PIC : ###
 * @par
 * -<b> MCU           </b> :      PIC18F87K22
 * -<b> Dev. Board    </b> :      EasyPIC Pro v7
 * -<b> Oscillator    </b> :      16 Mhz external
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for PIC 6.6.3
 *
 * ### Test configuration dsPIC : ###
 * @par
 * -<b> MCU           </b> :      dsPIC33EJ256GP710A
 * -<b> Dev. Board    </b> :      EasyPIC Fusion v7
 * -<b> Oscillator    </b> :      8 Mhz internal
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for dsPIC 6.2.1
 *
 * ### Test configuration AVR : ###
 * @par
 * -<b> MCU           </b> :      ATMEGA32
 * -<b> Dev. Board    </b> :      EasyAVR v7
 * -<b> Oscillator    </b> :      8 Mhz external
 * -<b> Ext. Modules  </b> :      CLICKNAME Click
 * -<b> SW            </b> :      MikroC PRO for FT90x 6.1.1
 */

/**
 * @mainpage
 * <h3> General Description </h3>
 * @par
 *
 *
 * <h3> Features </h3>
 * @par
 *
 *
 */

#ifndef STEPPER4_Lib_H
#define STEPPER_Lib_H

/******************************************************************************
* Includes
*******************************************************************************/

#include <stdint.h>
//#include <stdbool.h>               // koristim iskljucivo stdint +++
#include <math.h>
#include "__STEPPER4_HAL.h"

/******************************************************************************
* Preprocessor Constants
*******************************************************************************/
  #define true 1            // ovo sam dodao jer nemam vise bool                      // globalno dostupni def treba  _STEPPER4_true
  #define false 0
/******************************************************************************
* Configuration Constants
*******************************************************************************/

/******************************************************************************
* Macros
*******************************************************************************/

/******************************************************************************
* Typedefs
*******************************************************************************/

/**
 * @enum Motor type
 *
 *
 * @par
 * + FUNCTION - Use the functional interface, implementing your own driver
 * + DRIVER - Stepper Driver, 2 driver pins required
 * + FULL2WIRE - 2 wire stepper, 2 motor pins required
 * + FULL3WIRE - 3 wire stepper, such as HDD spindle, 3 motor pins required
 * + FULL4WIRE - 4 wire full stepper, 4 motor pins required
 * + HALF3WIRE - 3 wire half stepper, such as HDD spindle, 3 motor pins required
 * + HALF4WIRE - 4 wire half stepper, 4 motor pins required
 */
typedef enum
{
    STEPPER4_mif_FUNCTION                    = 0,     // imena clanova enum moraju da sadrze deo imena enuma - mif
    STEPPER4_mif_DRIVER                      = 1,     // globalno dostupni pa treba CIC prefiks
    STEPPER4_mif_FULL2WIRE                   = 2,
    STEPPER4_mif_FULL3WIRE                   = 3,
    STEPPER4_mif_FULL4WIRE                   = 4,
    STEPPER4_mif_HALF3WIRE                   = 6,
    STEPPER4_mif_HALF4WIRE                   = 8

} T_STEPPER4_motor_iface;  // T_ treba ispred, i CIC prefiks

/******************************************************************************
* Variables
*******************************************************************************/

/******************************************************************************
* Function Prototypes
*******************************************************************************/
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief <h3> HW Layer Initialization </h3>
 *
 * @par
 * Prepares library for usage and sets needed values to default.
 *
 * @note
 * This function must be called first.
 *
 * Example :
 * @code
 *
 * @endcode
 */
//

void STEPPER4_init
(
        T_STEPPER4_motor_iface ifacea,          // !!!!!!!!!!!!!!!!!!!! kako organizovati imena !!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        uint16_t mcu_freq,
        int8_t use_en_pin,    // bio je bool koji je u includovanoj bibioteci typedefvan u char pa koristim stdint char
        int8_t use_inv_pin,
        int8_t enable_motor
);

void STEPPER4_init_fp
(
        void ( *fpforward )(  ),                      //void ne moze kao argument
        void ( *fpbackward )(  )                      // dodajem fp
);

/**
 * @brief Set the target position
 *
 * @par
 * The @link stepper_run @endlink function will try to move the motor
 * ( at most one step per call ) from the current position to the
 * target position set by the most recent call to this function.
 *
 * @note
 * This function also recalculates the speed for the next step. If you are
 * trying to use constant speed movements, you should call
 * @link stepper_set_speed @endlink after calling this one.
 *
 * @param[in] absolute - The desired absolute position. Negative is anticlockwise
 * from the 0 position.
 */
void STEPPER4_move_to(int32_t absolut);

/**
 * @brief Set the target position relative to the current position
 *
 * @param[in] relative - The desired position relative to the current position.
 * Negative is anticlockwise from the current position.
 */
void STEPPER4_move(int32_t relative);

/**
 * @brief stepper_run
 *
 * @par
 * Poll the motor and step it if a step is due, implementing accelerations and
 * decelerations to acheive the target position. You must call this as
 * frequently as possible, but at least once per minimum step time interval,
 * preferably in your main loop.
 *
 * @note
 * Each call will make at most one step, and then only when a step is due,
 * based on the current speed and the time since the last step.
 *
 * @return
 */
int8_t STEPPER4_run( );

/**
 * @brief Poll the motor and step it
 *
 * @par
 * Only if a step is due, implementing a constant speed
 * as set by the most recent call to @link stepper_set_speed @endlink.
 * You must call this as frequently as possible, but at least once per step
 * interval.
 *
 * @return
 */
int8_t STEPPER4_run_speed();

/**
 * @brief Distance
 *
 * @par
 * The distance from the current position to the target position in steps.
 * Positive is clockwise from the current position.
 */
int32_t STEPPER4_measure_Distancetogo ( );   // ubacio sam measure jer mora glagol posle prefiksa

/**
 * @brief The most recently set target position.
 *
 * @par
 * Positive is clockwise from the 0 position in steps.
 *
 * @return
 */
int32_t STEPPER4_read_target_pos( );   // dodao read

/**
 * @brief The currently motor position.
 *
 * @par
 * Positive is clockwise from the 0 position in steps.
 *
 * @return
 */
int32_t STEPPER4_read_current_pos( );       // dodao read

/**
 * @brief Resets the current position of the motor
 *
 * @par
 * Wherever the motor happens to be right now is considered to be the new
 * 0 position. Useful for setting a zero position on a stepper after an initial
 * hardware positioning move.
 *
 * @note
 * Has the side effect of setting the current motor speed to 0.
 *
 * @param[in] position
 */
void STEPPER4_set_current_position(int32_t position);

/**
 * @brief Sets Maximum Speed
 *
 * @par
 * The @link stepper_run @endlink function will accelerate up to the speed set
 * by this function.
 *
 * @note
 * The maximum speed achievable depends on your processor and clock speed.
 *
 * @warning
 * Speeds that exceed the maximum speed supported by the processor may result
 * in non-linear accelerations and decelerations.
 *
 * @param[in] speed - The desired maximum speed in steps per second. Must be
 * greater than 0.
 */
void STEPPER4_set_max_speed(float speed);

/**
 * @brief Maximum Speed
 *
 * @par
 * Returns the maximum speed configured for this stepper that was previously
 * set by @link stepper_set_max_speed @endlink
 *
 * @return
 */
float STEPPER4_read_max_speed( );    // dodao read

/**
 * @brief Set Acceleration
 *
 * @par
 * Sets the acceleration / deceleration rate.
 *
 * @note
 * This is an expensive call since it requires a square root to be calculated.
 * Dont call more ofthen than needed.
 *
 * @param[in] acceleration - The desired acceleration in steps per second per
 * second. Must be greater than 0.
 */
void STEPPER4_set_acc(float acceleration);

/**
 * @brief Set Speed
 *
 * @par
 * Constant speed for use with @link stepper_run_speed @endlink.
 *
 * @param[in] speed - The desired constant speed in steps per second.
 * Positive is clockwise. Speeds of more than 1000 steps per second are
 * unreliable. Very slow speeds may be set.
 */
void STEPPER4_set_speed(float speed);

/**
 * @brief Current Speed
 *
 * @par
 * The most recently set speed.
 *
 * @return
 */
float STEPPER4_read_speed( );  // dodao read

//void stepper_enable_outputs

/**
 * @brief Distance
 *
 * @par
 * The distance from the current position to the target position in steps.
 * Positive is clockwise from the current position.
 */
int32_t STEPPER4_measure_dist( );// dodao measure, proveriti sa measure distance to go funkcijom gore da li je ista!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

/**
 * @brief The most recently set target position.
 *
 * @par
 * Positive is clockwise from the 0 position in steps.
 *
 * @return
 */
int32_t STEPPER4_read_tar_pos( ); // dodao read

//void stepper_disable_outputs

/**
 * @brief The currently motor position.
 *
 * @par
 * Positive is clockwise from the 0 position in steps.
 *
 * @return
 */
int32_t STEPPER4_read_cur_pos( );  // dodao read, proveriti da li je gore duplirana fja !!!!!!!!!!!!!!!!!!!!!!

/**
 * @brief Resets the current position of the motor
 *
 * @par
 * Wherever the motor happens to be right now is considered to be the new
 * 0 position. Useful for setting a zero position on a stepper after an initial
 * hardware positioning move.
 *
 * @note
 * Has the side effect of setting the current motor speed to 0.
 *
 * @param[in] position
 */
void STEPPER4_set_cur_pos(int32_t position);     // proveriti dupliranje !!!!!!!!!!!!!!!!!!!!!!!!!!

/**
 * @brief Move the motor
 *
 * @par
 * Uses acceleration and deceleration to move the motor to target position and
 * blocks until it is at position.
 *
 * @note
 * Dont use this in event loops, since it blocks.
 */
void STEPPER4_run_to_position( );

/**
 * @brief Runs at the Currently Selected Speed
 *
 * @par
 * Does not uses the acceleration and deceleration.
 */
int8_t STEPPER4_run_speed_to_position( );

/**
 * @brief Runs the Motor to New Position
 *
 * @par
 * Moves the motor ( with acceleration / deceleration ) to the new target
 * position and blocks until it is at position.
 *
 * @note
 * Dont use this in event loops, since it blocks.
 */
void STEPPER4_run_to_new_position (int32_t position);

/**
 * @brief Stops the motor
 *
 * @par
 * Sets a new target position that causes the stepper to stop as quickly as
 * possible, using the current speed and acceleration parameters.
 */
void STEPPER4_stop( );

/**
 * @brief Sets the minimum pulse width allowed by the stepper driver
 *
 * @par
 * The minimum practical pulse width is approximately 20 microseconds. Times
 * less than 20 microseconds will usually result in 20 microseconds or so.
 *
 * @param[in] min_width - The minimum pulse width in microseconds
 */
void STEPPER4_set_min_pulse_width (uint16_t width);   // izbacio min da biargument bio jedna rec

/**
 * @brief Motor Running
 *
 * @par
 * Checks to see if the motor is currently running to a target.
 */
int8_t STEPPER4_is_running( );

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* STEPPER4_Lib_H */
/*** End of File **************************************************************/